package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.ETC_MODBUS_TYPE)
public class EtcPersistenceHandler extends AbstractPersistenceHandler {

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private SnowflakeConfig snowflakeConfig;
    @Resource(name = "stringRedisTemplate")
    private RedisTemplate redisTemplate;

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_ETC.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_ETC.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, EtcPersistenceHandler.this.snowflakeConfig.snowflakeId());
                EtcPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                EtcPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 5, "deviceTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 6, "runMode", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 7, "wellStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 8, "plugStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 9, "statusOpenTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 10, "statusCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 11, "oilPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 12, "casPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 13, "prePressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 14, "pipePressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 15, "liquidLevel", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 16, "temperature", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 17, "humidity", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 18, "opmode", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 19, "timer1", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 20, "timer2", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 21, "timer1Open", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 22, "timer1Close", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 23, "timer2Open", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 24, "timer2Close", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 25, "timingOpen", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 26, "timingClose", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 27, "timingCelay", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 28, "presource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 29, "pressureOpen", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 30, "pressureClose", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 31, "triggerType", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 32, "stabilityTime", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 33, "maxOpenWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 34, "maxCloseWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 35, "minOpenWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 36, "minCloseWell", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 37, "presproTect", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 38, "presproSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 39, "openWellLimitMax", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 40, "openWellLimitMin", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 41, "plugInitStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 42, "plugSustainTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 43, "plugCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 44, "tpInitStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 45, "tpOpenSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 46, "tpOpenTrigger", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 47, "tpOpenPressure", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 48, "tpOpenTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 49, "tpCloseSource", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 50, "tpCloseTrigger", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 51, "tpClosePressure", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 52, "tpCloseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 53, "tpStabilityTime", Types.INTEGER);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }
}
